# -*- coding: iso-8859-1 -*-
import random
import os, sys
from os import listdir
from os.path import isfile, join
import pyarrow as pa
import pandas as pd
import datasets
from database_preparation.utils_stringpreparation import read_german_text
import argparse

# parse arguments:
sys.path.append(os.getcwd())
parser = argparse.ArgumentParser()
parser.add_argument("--path_to_reports",
                    default='../DataNephroTexts/reports')
parser.add_argument("--output_path",
                    default='./LanguageModelling/hf_nephro_set_1')
parser.add_argument("--percentage_train_amount", type=float, default=0.9)
args = parser.parse_args()

def save_as_hf_dataset(datapath, file_id_list, output_path):
    report_texts = []
    for id in file_id_list:
        text = (read_german_text(datapath + str("/") + str(id)))
        report_texts.append(text)
    df = pd.DataFrame({
        'text': report_texts
    })
    mytable = pa.Table.from_pandas(df)
    my_dataset = datasets.Dataset(mytable)
    #my_dataset.save_to_disk(output_path)
    my_dataset.to_json(output_path + ".json")
    print(f"Generated {output_path}")

def main():

    print("processing " + args.path_to_reports)

    reports = [f for f in listdir(args.path_to_reports) if isfile(join(args.path_to_reports, f))]

    reps0 = [r for r in reports if r[-5] == '0']
    random.shuffle(reps0)
    last_index = len(reps0) - 1
    until = int(args.percentage_train_amount * last_index)

    if args.percentage_train_amount < 1:
        train = [e for i, e in enumerate(reps0) if i <= until]
        val = [e for i, e in enumerate(reps0) if i > until]
        save_as_hf_dataset(args.path_to_reports, train, args.output_path + "_train")
        save_as_hf_dataset(args.path_to_reports, val, args.output_path + "_validation")
    else:
        save_as_hf_dataset(args.path_to_reports, reps0, args.output_path)


    # how to load dataset:
    '''ds = datasets.load_from_disk("./LanguageModelling/path2set")
    print(ds)'''

    return 0


if __name__ == "__main__":
    main()
