
#%% argsparse section
import sys, os
sys.path.append(os.getcwd())

from TextClustering.argsparse_clustering_preamble import argsparse_preamble
args = argsparse_preamble()

from database_preparation.utils_labeled_datasets import is_text_lst_tokenized, is_text_lst_tfidf_vectorized
import pickle
from sklearn.feature_extraction.text import TfidfVectorizer
import matplotlib.pyplot as plt
import pandas as pd
import umap
import hdbscan
import numpy as np
from nltk import RegexpTokenizer
from TextClustering.utils_metrics import ClusterMetrics
from database_preparation.preprocess import print_meta_data
from TextClassification.classification_for_cluster_evaluation import cross_validate_label_corpus_with_simple_SVM


tokenizer = RegexpTokenizer(r'\w+')

#%% load the data
with open(args.path2corpus, 'rb') as f:
    diag_lst = pickle.load(f)

print_meta_data(args.path2corpus)

def identity(word):
    return word

text_is_vectorized = is_text_lst_tfidf_vectorized(args.path2corpus)
if not (is_text_lst_tokenized(args.path2corpus) or text_is_vectorized):
    print("Error: " + args.path2corpus + ' has to be a list of tokenized texts or tfidf-vectorized texts!')
    exit(1)

def create_vectorizer(data):
    vec = TfidfVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
    vec = vec.fit_transform(data)
    return vec

if text_is_vectorized:
    text_features = diag_lst
else:
    text_features = create_vectorizer(diag_lst)

#%% perform umap for dimension-reduction (for cluster-detection)
umap_text_features = umap.UMAP(n_neighbors=15,
                            n_components=5,
                            metric='cosine').fit_transform(text_features)

# and perform umap-dimension-reduction for visulatizaton
umap_text_features2D = umap.UMAP(n_neighbors=15,
                                 n_components=2,
                                 min_dist=0.0, metric='cosine').fit_transform(text_features)

if args.find_k_value:
    # %% perform hdbscan for cluster-dectection with different cluster sizes to find a good solution...by eye..
    list_cluster_size = [int(k) for k in np.arange(3, 23, 1)]
    s_score, n_cluster, svm_scores = [], [], []
    for i_cluster_size in list_cluster_size:

        cluster = hdbscan.HDBSCAN(min_cluster_size=i_cluster_size,
                                  metric='euclidean',
                                  cluster_selection_method='eom').fit(umap_text_features)


        result = pd.DataFrame(umap_text_features2D, columns=['x', 'y'])
        result['labels'] = cluster.labels_.tolist() # cluster.labels_

        print(np.unique(result.labels))

        #%% Visualize clusters
        outliers = result.loc[result.labels == -1, :]
        clustered = result.loc[result.labels != -1, :]
        clustered['labels'] = [str(i) for i in clustered['labels']]

        evaluation = ClusterMetrics(umap_text_features, cluster.labels_.tolist())
        s_score.append(evaluation.s_score)
        svm_scores.append(
            cross_validate_label_corpus_with_simple_SVM(cluster.labels_.tolist(), args.path2corpus + '.pkl',
                                                        False))
        n_cluster.append(len(np.unique(cluster.labels_.tolist())))

    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()
    ax3 = ax1.twinx()
    ax1.plot(list_cluster_size, s_score, 'bx-')
    ax2.plot(list_cluster_size, n_cluster, 'rx-')
    ax3.plot(list_cluster_size, svm_scores, 'gx-')
    ax1.set_xlabel('Minimal cluster size')
    ax1.yaxis.label.set_color('blue')
    ax1.set_ylabel('s-score')
    ax2.yaxis.label.set_color('red')
    ax2.set_ylabel('Number of clusters')
    ax3.yaxis.label.set_color('green')
    ax3.set_ylabel('svm accuracy')
    plt.title('Ellbow-method-like plot')
    plt.savefig("TextClustering/plots/elbow_method/HDBSCAN_elbow_plot.png", dpi=300)
    plt.show()

    exit()

#%% perform hdbscan with best cluster size
cluster = hdbscan.HDBSCAN(min_cluster_size=args.k_value,
                              metric='euclidean',
                              cluster_selection_method='eom').fit(umap_text_features)


result = pd.DataFrame(umap_text_features2D, columns=['x', 'y'])
result['labels'] = cluster.labels_.tolist() # cluster.labels_
clusters = np.int8([str(i) for i in result['labels']])
outliers = result.loc[result.labels == -1, :]
clusters_no_outliers = result.loc[result.labels != -1, :]

unique_clusters = np.unique(result.labels)
print(f"\nfound {len(unique_clusters[unique_clusters>-1])} clusters.\n")

# save umaped vectors:
df = pd.read_pickle(args.df_cases_file)
df['umapX_HDBSCAN'] = umap_text_features2D[:, 0]
df['umapY_HDBSCAN'] = umap_text_features2D[:, 1]
df['label_HDBSCAN'] = clusters
df.to_pickle(args.df_cases_file)


#%% and evaluate the results with several metrics (not needing ground truth)
evaluation = ClusterMetrics(umap_text_features[result.labels >= 0,], clusters_no_outliers.labels.tolist(),
                            file_name= "TextClustering/cluster_metrics/HDBSCAN_metrics.pkl")
evaluation.write_to_file()
