# %% argsparse preamble
import sys, os

sys.path.append(os.getcwd())

from TextClustering.argsparse_clustering_preamble import argsparse_preamble

args = argsparse_preamble()

from database_preparation.utils_labeled_datasets import is_text_lst_tokenized

if is_text_lst_tokenized(args.path2corpus):
    print("Error: " + args.path2corpus + ' is tokenized! '
                                         'Please pass texts list where each text is a single string!')
    exit(1)

# %% prepare the background
import pickle
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import umap
import hdbscan
from TextClustering.utils_metrics import ClusterMetrics
from sentence_transformers import SentenceTransformer
from database_preparation.preprocess import print_meta_data


embedding_backup_folder = "database/backup_files/"
if not os.path.isdir(embedding_backup_folder):
    os.makedirs(embedding_backup_folder)
path_2_pathoBERT = "./LanguageModelling/ger-patho-bert-2"

with open(args.path2corpus, 'rb') as f:
    diag_lst = pickle.load(f)



print_meta_data(args.path2corpus)

if args.do_embedding:

    # %% load the model
    if args.model2use == "German_BERT":
        model = SentenceTransformer("Sahajtomar/German-semantic")
    elif args.model2use == "Patho_BERT":
        model = SentenceTransformer(path_2_pathoBERT)

    # %% and apply the embedding-model to the text (only once, since very time-consuming)
    if not 'embeddings' in locals():
        embeddings = model.encode(diag_lst, show_progress_bar=True)
        np.save(embedding_backup_folder + args.model2use + "_embeddingsBackup.npy", embeddings)

# %% load it (if not there)
if not 'embeddings' in locals():
    embeddings = np.load(embedding_backup_folder + args.model2use + "_embeddingsBackup.npy")

# %% perform umap
umap_embeddings = umap.UMAP(n_neighbors=15,
                            n_components=5,
                            metric='cosine').fit_transform(embeddings)

# and perform umap-dimension-reduction for visulatizaton
umap_text_features2D = umap.UMAP(n_neighbors=15,
                                 n_components=2,
                                 min_dist=0.0, metric='cosine').fit_transform(embeddings)

if 'umap_embeddings' in locals():
    np.save(embedding_backup_folder + args.model2use + "_umap_embeddingsBackup.npy", umap_embeddings)

# %% perform repetitive clustering to find the best min_cluster:size
if not 'umap_embeddings' in locals():
    umap_embeddings = np.load(embedding_backup_folder + "umap_embeddingsBackup.npy")

if args.find_k_value:
    cluster_size = range(5, 40, 2)
    s_score, n_cluster = [], []
    for i_cluster_size in cluster_size:
        cluster = hdbscan.HDBSCAN(min_cluster_size=i_cluster_size,
                                  metric='euclidean',
                                  cluster_selection_method='eom').fit(umap_embeddings)

        # and evaluate the results with several metrics (not needing ground truth)
        from TextClustering.utils_metrics import ClusterMetrics

        evaluation = ClusterMetrics(umap_embeddings, cluster.labels_.tolist())
        s_score.append(evaluation.s_score)
        n_cluster.append(len(np.unique(cluster.labels_.tolist())))

    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()
    ax1.plot(cluster_size, s_score, 'bx-')
    ax2.plot(cluster_size, n_cluster, 'rx-')
    ax1.set_xlabel('Minimal cluster size')
    ax1.yaxis.label.set_color('blue')
    ax1.set_ylabel('Silhouette Coefficient')
    ax2.yaxis.label.set_color('red')
    ax2.set_ylabel('Number of clusters')
    plt.title('Ellbow-method-like plot')
    plt.show()
    sys.exit()

# %% perform hdbscan-clustering
if not 'umap_embeddings' in locals():
    umap_embeddings = np.load(embedding_backup_folder + "umap_embeddingsBackup.npy")

cluster = hdbscan.HDBSCAN(min_cluster_size=args.k_value,
                          metric='euclidean',
                          cluster_selection_method='eom').fit(umap_embeddings)

# and print the results
result = pd.DataFrame(umap_text_features2D, columns=['x', 'y'])
result['labels'] = cluster.labels_.tolist()  # cluster.labels_
print("cluster indices: " + str(np.unique(result.labels)))
outliers = result.loc[result.labels == -1, :]
clustered = result.loc[result.labels != -1, :]

print(str(len(outliers.x)) + " outliers")

# save umaped vectors:
df = pd.read_pickle(args.df_cases_file)
df['umapX_' + args.model2use] = result.x
df['umapY_' + args.model2use] = result.y

# %% update df_cases
df['label_' + args.model2use] = result.labels
df.to_pickle(args.df_cases_file)

# %% and evaluate the results with several metrics (not needing ground truth)
evaluation = ClusterMetrics(umap_embeddings[result.labels >= 0,], clustered.labels.tolist(),
                            file_name="TextClustering/cluster_metrics/" + args.model2use + "_metrics.pkl")
evaluation.write_to_file()
