import sys, os
from tqdm import tqdm
import pandas as pd
import pickle
from database_preparation.preprocess import print_meta_data
from database_preparation.utils_labeled_datasets import is_text_lst_tokenized
sys.path.append(os.getcwd())

# parse arguments:
from TextClustering.argsparse_clustering_preamble import argsparse_preamble
args = argsparse_preamble()

print("arguments:")
print(args)
if is_text_lst_tokenized(args.path2corpus):
    print("Error: "+args.path2corpus + '.pkl is tokenized! '
            'Please pass texts list where each text is a single string!')
    exit(1)

#%% load the data
with open(args.path2corpus, 'rb') as f:
    diag_lst = pickle.load(f)


print_meta_data(args.path2corpus)

#%% perform clustering repetitive
if args.find_k_value:

    from TextClustering.utils_metrics import ClusterMetrics
    from TextClustering.basedOn_Embedding.top2vec import Top2Vec
    import matplotlib.pyplot as plt # load our modified version (for visualization)

    s_score, n_cluster = [], []
    cluster_size = range(3, 25, 2)

    for i_cluster_size in tqdm(cluster_size):

        #%% perform text-clustering (like in the paper)
        hdbscan_args = {'min_cluster_size': i_cluster_size,
                        'metric': 'euclidean',
                        'cluster_selection_method': 'eom'}
        model = Top2Vec(diag_lst,
                        embedding_model=args.model2use,
                        min_count=0,
                        hdbscan_args=hdbscan_args)

        #%% get the clusters
        n_cluster.append(model.get_num_topics())
        evaluation = ClusterMetrics(model.umap_model.embedding_[model.result.labels >= 0,],
                                    model.clustered.labels.tolist())
        s_score.append(evaluation.s_score)

    #%% plot the results
    fig, ax1 = plt.subplots()
    ax2 = ax1.twinx()
    ax1.plot(cluster_size, s_score, 'bx-')
    ax2.plot(cluster_size, n_cluster, 'rx-')
    ax1.set_xlabel('Minimal cluster size')
    ax1.yaxis.label.set_color('blue')
    ax1.set_ylabel('Silhouette Coefficient')
    ax2.yaxis.label.set_color('red')
    ax2.set_ylabel('Number of clusters')
    plt.title('Ellbow-method-like plot')
    plt.show()
    exit()

#%% perform text-clustering (like in the paper)
from TextClustering.basedOn_Embedding.top2vec import Top2Vec # load our modified version (for visualization)

hdbscan_args = {'min_cluster_size': args.k_value,
                'metric': 'euclidean',
                'cluster_selection_method': 'eom'}
model = Top2Vec(diag_lst,
                embedding_model = args.model2use,
                min_count = 0,
                hdbscan_args=hdbscan_args)

#%% get the words and topics
# by their way
model.get_num_topics()
topic_sizes, topic_nums = model.get_topic_sizes()

# print some infos:
outliers = model.umap_model.embedding_[model.result.labels == -1,]
print(f"found {len(topic_nums)} topics")
print(f"found {len(outliers)} outilers.")

topic_words, word_scores, topic_nums = model.get_topics(len(topic_nums))
pd.DataFrame(topic_words).to_excel(
    'TextClustering/tables/WordsPerCluster_Top2Vec_' + args.model2use  + '.xlsx',
    sheet_name= "in-function")

# save umaped vectors and labels:
df = pd.read_pickle(args.df_cases_file)
df['umapX_top2vec'] = model.result.x
df['umapY_top2vec'] = model.result.y
df['label_top2vec'] = model.result.labels
df.to_pickle(args.df_cases_file)

#%% calculate clustering-metrics
from TextClustering.utils_metrics import ClusterMetrics
evaluation = ClusterMetrics(model.umap_model.embedding_[model.result.labels >= 0,], model.clustered.labels.tolist(),
                            file_name= "TextClustering/cluster_metrics/top2vec_metrics.pkl")
evaluation.write_to_file()