import pandas as pd
from TextClustering.utils_metrics import cluster_scatter_plot
import numpy as np
from database_preparation.utils_labeled_datasets import label_list_as_int_list

clustersets = ["GSDPMM", "KMeans", "LDA", "HDBSCAN",
               "top2vec", "Patho_BERT", "German_BERT"]
plot_titles = ["GSDPMM (UMAP representation)", "k-means (UMAP representation)",
               "LDA (UMAP representation)", "HDBSCAN (UMAP representation)",
               "top2vec (UMAP representation)", "Patho-BERT (UMAP representation)",
               "German-BERT (UMAP representation)"]
df_cases_file = "database/df_cases.pkl"

def save_umap_plot(clustersetname, df, title=None):
    if not 'label_' + clustersetname in df:
        print("skipping " + clustersetname + ", it is not in df_cases_file.")
        return
    predictedCluster_text_features = label_list_as_int_list(df['label_' + clustersetname])

    try:
        umap_text_features2D = np.asarray([[e for e in df['umapX_' + clustersetname]],
                                           [e for e in df['umapY_' + clustersetname]]])
    except:
        print("there is no umapX_" + clustersetname + " in database/df_cases.pkl. => skipping")
        return
    umap_text_features2D = np.transpose(umap_text_features2D)

    cluster_scatter_plot(umap_text_features2D, predictedCluster_text_features,
                 "TextClustering/plots/UMAP/" + clustersetname + "_UMAP.png",
                         show_plot=False, colorblindfriendly=False, fig_title=title)
    if 'label_author' in df:
        author_labels = df["label_author"]
        cluster_scatter_plot(umap_text_features2D, author_labels,
                     "TextClustering/plots/UMAP/" + clustersetname + "_UMAP_authors.png",
                             show_plot=False, colorblindfriendly=True, number_data_points=False
                             , fig_title=title + ", colored by authors")
    if 'label_golden' in df:
        golden_labels = df["label_golden"]
        cluster_scatter_plot(umap_text_features2D, golden_labels,
                     "TextClustering/plots/UMAP/" + clustersetname + "_UMAP_goldenlabel.png",
                             show_plot=False, colorblindfriendly=False
                             , fig_title=title + " colored with golden labels")

def main():
    df = pd.read_pickle(df_cases_file)
    for clustersetname in clustersets:
        if clustersetname in clustersets:
            title = plot_titles[clustersets.index(clustersetname)]
        else:
            title = None
        save_umap_plot(clustersetname, df, title)

    # plot author-colored and cluster-colored lda clustersets as pca representation:
    if 'label_LDA' in df and 'pcaX_LDA' in df:
        predictedCluster_text_features = df['label_LDA']

        features2D = np.asarray([[e for e in df['pcaX_LDA']],
                                 [e for e in df['pcaY_LDA']]])
        features2D = np.transpose(features2D)

        cluster_scatter_plot(features2D, predictedCluster_text_features,
                     "TextClustering/plots/PCA/LDA_PCA.png",
                             show_plot=False, colorblindfriendly=False,
                             fig_title="LDA (PCA representation)")
        if 'label_author' in df:
            cluster_scatter_plot(features2D, df["label_author"],
                                 "TextClustering/plots/PCA/LDA_PCA_authors.png",
                                 show_plot=False, colorblindfriendly=True,
                                 number_data_points=False, fig_title='LDA (PCA representation), colored by authors')

if __name__ == '__main__':
    main()
