# -*- coding: iso-8859-1 -*-

from database_preparation.utils_stringpreparation import get_most_frequent_words
import numpy as np
import pandas
import yake
import nltk
import openpyxl as pxl
from database_preparation.utils_labeled_datasets import text_label_2_labeled_dataset
import pandas as pd
from database_preparation.utils_labeled_datasets import label_list_as_int_list
from database_preparation.stop_word_list import filter_stopwords


def get_nwordlist(text_lst, cluster_lst, n_words=10,
                  excel_file_path=[],
                  method="frequency-based",
                  filter_stop_words=True):
    """
    - if file_path given, the wordlist will be saved as .xlsx with sheet name = method
    """

    ignore_outlier = True

    # %% prepare parameter
    docs = {'text': text_lst, 'cluster': cluster_lst}
    n_cluster = np.unique(cluster_lst)
    frame = pandas.DataFrame(docs, index=[cluster_lst])

    text1 = np.asarray(text_lst[0])
    text_lst_is_tokenized = bool(text1.ndim)

    if method in ['svm-based', 'LR-based']:

        ###### prepare text data: ######
        if text_lst_is_tokenized:
            if filter_stop_words:
                dataset = text_label_2_labeled_dataset(
                    [filter_stopwords(text) for text in text_lst], cluster_lst
                )
            else:
                dataset = text_label_2_labeled_dataset(text_lst, cluster_lst)
        else:
            # print("passed text list is not tokenized. Tokenizing it now with nltk...")
            # tokenize
            tokenized_texts = []
            for t_text in text_lst:
                if filter_stop_words:
                    tokenized_texts.append(filter_stopwords(nltk.tokenize.word_tokenize(t_text, language='german')))
                else:
                    tokenized_texts.append(nltk.tokenize.word_tokenize(t_text, language='german'))
            dataset = text_label_2_labeled_dataset(tokenized_texts, cluster_lst)

        ###### train svm: ######
        from sklearn.pipeline import make_pipeline
        from sklearn.feature_extraction.text import TfidfVectorizer
        from sklearn.svm import LinearSVC
        from sklearn.linear_model import LogisticRegressionCV
        def identity(words):
            return words

        def get_trained_svm(texts, labels):
            vec = TfidfVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
            # svd = TruncatedSVD(n_components=1000, n_iter=7, random_state=42)
            # lsa = make_pipeline(vec, svd)
            # clf = SVC(probability=True, kernel="linear")
            clf = LinearSVC()
            # pipe = make_pipeline(lsa, clf)
            pipe = make_pipeline(vec, clf)
            pipe.fit(texts, labels)
            return pipe, vec, clf

        def get_trained_LR(text, labels):
            # vec = CountVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
            vec = TfidfVectorizer(tokenizer=identity, preprocessor=None, lowercase=False)
            clf = LogisticRegressionCV()
            pipe = make_pipeline(vec, clf)
            pipe.fit(text, labels)
            return pipe, vec, clf

        if method == 'svm-based':
            pipe, vec, clf = get_trained_svm(dataset['text'], dataset['label'])
        else:
            pipe, vec, clf = get_trained_LR(dataset['text'], dataset['label'])

        def get_correct_predictions(texts, labels, pipe):
            y_preds = pipe.predict(texts)
            correct_predictions = []
            for i, y_pred in enumerate(y_preds):
                if y_pred == labels[i]:
                    correct_predictions.append(i)
            return correct_predictions

        correct_predictions = get_correct_predictions(dataset['text'], dataset['label'], pipe)
        '''print("found " + str(len(correct_predictions)) + '/' +
              str(len(dataset['text'])) + " correct predicted docs.")'''

        del clf, vec, pipe
        if method == 'svm-based':
            pipe, vec, clf = get_trained_svm(dataset[correct_predictions]['text'],
                                             dataset[correct_predictions]['label'])
        else:
            pipe, vec, clf = get_trained_LR(dataset[correct_predictions]['text'],
                                            dataset[correct_predictions]['label'])

        '''print("using "+method+" which predicted " +
              str(len(
                  get_correct_predictions(dataset[correct_predictions]['text'], dataset[correct_predictions]['label'], pipe)))
              + '/' + str(len(dataset[correct_predictions]['text'])) + " documents correctly.")'''

        ########## weight analysis ##############
        # get feature importance:
        feature_names = vec.get_feature_names_out()
        top_word_lists = []
        for i, coef in enumerate(clf.coef_):
            if clf.__class__.__name__ == 'SVC':
                coef = coef.toarray()
                weights = list(zip(feature_names, [coef[0, i] for i in range(coef.shape[1])]))
            else:
                weights = list(zip(vec.get_feature_names_out(), coef))
            most_positives = sorted(weights, key=lambda x: -x[1])[:n_words]
            most_netatives = sorted(weights, key=lambda x: x[1])[:n_words]
            top_word_lists.append([tup[0] for tup in most_positives])

        '''f_mnames = vec.get_feature_names_out()
        coefs = clf.coef_.ravel()
        top_positive_coefficients = np.argsort(coefs)[-n_words:]
        top_negative_coefficients = np.argsort(coefs)[:n_words]
        top_coefficients = np.hstack([top_negative_coefficients, top_positive_coefficients])
        print(f_mnames[top_coefficients])'''

        save_topwordlist_as_excel(excel_file_path, top_word_lists, method)

        return top_word_lists

    # %% prepare different methods
    if method == "TFIDF-based":

        cluster_words = []
        clusters = list(np.unique(cluster_lst))

        if ignore_outlier:  # if yes: remove cluster -1
            if -1 in clusters:
                clusters.remove(-1)

        if text_lst_is_tokenized:
            for i_cluster in clusters:
                t_frame = frame[frame['cluster'] == i_cluster]
                if filter_stop_words:
                    t_text = []
                    for text in t_frame['text'].to_list():
                        filtered = filter_stopwords(text)
                        t_text.append(filtered)
                else:
                    t_text = t_frame['text'].to_list()
                words_of_cluster = [word for text in t_text for word in text]
                cluster_words.append(words_of_cluster)
        else:
            for i_cluster in clusters:
                t_frame = frame[frame['cluster'] == i_cluster]
                t_text = []
                for text in t_frame['text'].to_list():
                    tokenized = nltk.tokenize.word_tokenize(text, language='german')
                    if filter_stop_words:
                        t_text.append(filter_stopwords(tokenized))
                    else:
                        t_text.append(tokenized)

                words_of_cluster = [word for text in t_text for word in text]
                cluster_words.append(words_of_cluster)

    if method == "yake":
        def flatten(lst):
            text = ''
            for t_text in lst:
                text += str(t_text)
            return text

        kw_extractor = yake.KeywordExtractor(lan='German',
                                             n=1,
                                             dedupLim=0.9,
                                             dedupFunc='seqm',
                                             windowsSize=1,
                                             top=n_words, features=None)

    # %% perform it for TFIDF
    if method == "TFIDF-based":

        # %% prepare the corpus for tfidf
        from sklearn.feature_extraction.text import TfidfVectorizer
        vectorizer = TfidfVectorizer()
        corpus = [str(i) for i in cluster_words]
        vectors = vectorizer.fit_transform(corpus)
        names = vectorizer.get_feature_names_out()
        data = vectors.todense().tolist()
        df = pandas.DataFrame(data, columns=names)

        # %% create the words list
        word_list = []
        for i in df.iterrows():
            i_words = i[1].sort_values(ascending=False)[:n_words]
            word_list.append(i_words.index.to_list())

    else:

        # %% perform it for the other approaches
        clusters = []
        word_list = []

        for i_cluster in n_cluster:

            # ignore outlier
            if ignore_outlier:
                if i_cluster < 0:
                    continue

            t_frame = frame[frame['cluster'] == i_cluster]

            if method == "frequency-based":
                if text_lst_is_tokenized:
                    if filter_stop_words:
                        t_text = [filter_stopwords(text) for text in t_frame['text'].to_list()]
                    else:
                        t_text = t_frame['text'].to_list()
                else:
                    t_text = []
                    for text in t_frame['text'].to_list():
                        tokenized = nltk.tokenize.word_tokenize(text, language='german')
                        if filter_stop_words:
                            t_text.append(filter_stopwords(tokenized))
                        else:
                            t_text.append(tokenized)
                top_words = get_most_frequent_words(str(t_text), n_words)

            elif method == "yake":
                if text_lst_is_tokenized:
                    # join word-lists into single string

                    if filter_stop_words:
                        cluster_text_list = [filter_stopwords(text) for text in t_frame['text'].to_list()]
                    else:
                        cluster_text_list = t_frame['text'].to_list()
                    t_text = [' '.join(text) for text in cluster_text_list]
                else:
                    if filter_stop_words:  # tokenize->filterstopwords->join
                        t_text_tokenized = []
                        for text in t_frame['text'].to_list():  # tokenize+filtering
                            tokenized = nltk.tokenize.word_tokenize(text, language='german')
                            t_text_tokenized.append(filter_stopwords(tokenized))
                        t_text = [' '.join(text) for text in t_text_tokenized]  # join
                    else:
                        t_text = t_frame['text'].to_list()
                keywords = kw_extractor.extract_keywords(flatten(t_text))
                top_words = [i_key[0] for i_key in keywords]

            clusters.append(i_cluster)
            word_list.append(top_words)

    # %% how to get n_words in an array
    for i in range(0, len(word_list)):
        t_token = np.array(word_list[i])

        if len(t_token) < n_words + 1:
            t_token = np.append(t_token, np.repeat(np.nan, n_words - len(t_token)))

        t_token = t_token.reshape((1, -1))
        if i == 0:
            token_list = t_token
        else:
            token_list = np.concatenate((token_list, t_token), axis=0)

    # %% save it to the excel (bad style the way to the dark side is...)
    save_topwordlist_as_excel(excel_file_path, token_list, method)

    # %% return the results
    return word_list


def save_topwordlist_as_excel(file_path, token_list, sheet_name):
    if bool(file_path):
        try:  # if excelfile does exist, append new sheet to workbook:
            excel_book = pxl.load_workbook(file_path)
            if sheet_name in excel_book.get_sheet_names():
                excel_book.remove_sheet(sheet_name)
            with pandas.ExcelWriter(file_path, engine='openpyxl', if_sheet_exists=None) as writer:
                writer.book = excel_book
                writer.sheets = {
                    worksheet.title: worksheet
                    for worksheet in excel_book.worksheets
                }

                pandas.DataFrame(token_list).to_excel(writer, sheet_name)
                writer.save()

        except:  # otherwise: create new workbook and save
            pandas.DataFrame(token_list).to_excel(file_path, sheet_name=sheet_name)


def generate_save_topicwords(predictedClusters, text_lst, save_excel_file_path,
                             n_words=10, print_latex_table=False,
                             extraction_methods=['TFIDF-based', 'frequency-based', 'yake', 'svm-based'],
                             filter_stop_words=True):
    '''
    creates n_words most relevant topic-words with tfifg, tf and yake
    and saves it as .xlsx and as latex table
    '''

    latextable = ''
    for method in extraction_methods:
        word_list = get_nwordlist(text_lst, predictedClusters,
                                  n_words=n_words,
                                  excel_file_path=save_excel_file_path,
                                  method=method,
                                  filter_stop_words=filter_stop_words)

        latextable = latextable + "\n%================== " + save_excel_file_path + " " + method + " ================="
        latextable = latextable + get_top_cluster_words_as_latex_table(word_list)
        latextable = latextable + "%========================================================\n"

    if print_latex_table:
        print(latextable)
    with open(save_excel_file_path.replace('.xlsx', '') + '_latex.txt', 'w') as f:
        f.write(latextable)


def get_top_cluster_words_as_latex_table(words_lists, colors=None, cluster_topics=None):
    first_part = '''
%%%%%%%%%%%%%%%%%%%%%% LABEL %%%%%%%%%%%%%%%%%%%%%%%%%%
\\begin{table}[!htb] 
\caption{DESCRIPTON}\label{LABEL}
\\noindent
\\begin{tabularx}{\linewidth}{|@{}>{}l|@{\hspace{.5em}}X@{}|}
\hline
\\textbf{ cluster index - cluster name }  & \\textbf{keywords according to EXTRACTIONMETHOD} \\\\ \\hline
'''
    end_part = '''
\end{tabularx}
\end{table}
'''
    if colors is None or cluster_topics is None:
        latex_code_m = ""
        for i, words in enumerate(words_lists):
            latex_code_m = latex_code_m + str(i) + "  & " + ", ".join(words) + " \\\\ \hline \n"
    else:
        latex_code_m = ""
        colored_tops = [int(float(top[0])) if top[1] is None else '\\colorbox{' + top[1] + '}{' + top[0] + '}' for top in
                        cluster_topics]
        for i, words in enumerate(words_lists):
            colored_words = [word if colors[i][j] is None else '\\colorbox{'+colors[i][j]+'}{'+word+'}' for j,word in enumerate(words)]
            latex_code_m = latex_code_m + str(colored_tops[i]) + "  & " + ", ".join(colored_words) + " \\\\ \hline"
            if i+1 < len(words_lists):
                latex_code_m = latex_code_m + '\n'

    latex_code = first_part + latex_code_m + end_part
    latex_code = latex_code.replace("_", "\_")

    # print("================== cluster topics " + cluster_method + " =================")
    # print(latex_code)
    # print("========================================================")
    return latex_code


def print_excel_topciwordlist_as_latex(cluster_set, excel_file_path=None, topic_word_method="TFIDF-based"):
    if excel_file_path == None:
        excel_file_path = 'TextClustering/tables/WordsPerCluster_' + cluster_set + '_temp.xlsx'

    df_cases = pd.read_pickle("database/df_cases.pkl")

    # convert nan-values in int(-1):
    clusters = label_list_as_int_list(df_cases['label_' + cluster_set])

    df = pd.read_excel(open(excel_file_path, 'rb'), sheet_name=topic_word_method).T
    df.drop(['Unnamed: 0'], inplace=True)
    rename_dic = {}
    for i in range(np.max(clusters) + 1):
        rename_dic[i] = ' ' + str(i)

    df.rename(columns=rename_dic, inplace=True)

    latex_code = '''
    \\begin{table}[h]
    \caption{DESCRIPTON}\label{table_cluster_topics}
    \\noindent
    \\begin{tabularx}{\linewidth}{|@{}>{\\bfseries}l|@{\hspace{.5em}}X@{}|}
    \hline

    '''
    latex_code_m = ""
    for k in rename_dic.values():
        latex_code_m = latex_code_m + "\\textbf{" + str(k) + "}  & " + ", ".join(df[k]) + " \\\\ \hline \n"

    latex_code = latex_code + latex_code_m + '''
    \end{tabularx}
    \end{table}
    '''
    latex_code = latex_code.replace("_", "\_")

    print("%================== cluster topics of cluster-set" + cluster_set + " =================")
    print(latex_code)
    print("%========================================================")


def main():
    pass


if __name__ == '__main__':
    main()
